using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using NiVE2.Plugin.Interface;
using NiVE2.Drawing;
using NiVE2.Utils;
using NiVE2.Utils.Cache;
using ruche.aviutl.plugins;

namespace ruche.nive2.aviutlPluginPorts
{
    /// <summary>
    /// AviUtl̓vOC𗘗pNiVEI[fBI̓vOCNXB
    /// MediaInput ō쐬B
    /// </summary>
    internal class AudioMediaInput : InputChild
    {
        /// <summary>
        /// Tv[gB
        /// </summary>
        private static uint SampleRate = ReferenceData.WaveFormat.nSamplesPerSec;

        /// <summary>
        /// eCX^XB
        /// </summary>
        private InputBase _parent = null;

        /// <summary>
        /// ̓vOCC^tF[XB
        /// </summary>
        private InputPlugin _plugin = null;

        /// <summary>
        /// ʔԍ
        /// </summary>
        private int _childCode = 0;

        /// <summary>
        /// ĐԁB
        /// </summary>
        private double _length = 0.0;

        /// <summary>
        /// RXgN^B
        /// </summary>
        /// <param name="parent">eCX^XB</param>
        /// <param name="plugin">
        /// fBAt@Cǂݍݍς݂̓̓vOCB
        /// </param>
        /// <param name="id">q̎IDB</param>
        public AudioMediaInput(
            InputBase parent, InputPlugin plugin, int id)
        {
            if (parent == null)
            {
                throw new NullReferenceException("parent  null łB");
            }
            if (plugin == null)
            {
                throw new NullReferenceException("plugin  null łB");
            }
            if (!plugin.IsMediaFileOpened)
            {
                throw new ArgumentException(
                    "fBAt@CJĂ܂B", "plugin");
            }

            // p[^ێ
            _parent = parent;
            _plugin = plugin;
            _childCode = id;

            // vl̕ێ
            IInputInfo info = _plugin.MediaFileInfo;
            _length = (double)info.InputAudioSampleCount / SampleRate;
        }

        /// <summary>
        /// ͂̊JnsB
        /// </summary>
        void BeginInput()
        {
            // vOĈfBNgֈړ
            CurrentDirectoryStack.Instance.Push(
                Path.GetDirectoryName(_plugin.PluginFilePath));
        }

        /// <summary>
        /// ͂̏IsB
        /// </summary>
        void EndInput()
        {
            // JgfBNg
            CurrentDirectoryStack.Instance.Pop();
        }

        #region InputChild o

        public override int ChildCode
        {
            get { return _childCode; }
        }

        public override string MediaName
        {
            get
            {
                return Path.GetFileName(FileName) + "(audio)";
            }
        }

        public override bool HaveVideo
        {
            get { return false; }
        }

        public override bool HaveAudio
        {
            get { return true; }
        }

        public override bool IsImage
        {
            get { return false; }
        }

        public override double Length
        {
            get { return _length; }
        }

        public override InputBase Parent
        {
            get { return _parent; }
        }

        public override MediaInfo GetMedaiInfo()
        {
            return new MediaInfo(
                _length,
                SampleRate,
                false,
                true,
                false,
                Size.Empty);
        }

        public override PropertyBase[] GetDefaultProperty()
        {
            return new PropertyBase[] { };
        }

        public override PropertyEditControlBase[] GetControl()
        {
            return new PropertyEditControlBase[] { };
        }

        public override RectangleF GetBounds(
            double time, IComposition composition, ReadOnlyDictionary<string, PropertyBase> property)
        {
            // Ă΂Ȃ͂ꉞ
            return RectangleF.Empty;
        }

        public override NBitmap GetImage(
            double time, IComposition composition, ReadOnlyDictionary<string, PropertyBase> property)
        {
            // Ă΂Ȃ
            throw new NotImplementedException();
        }

        public override byte[] GetAudio(double time, int length, IComposition composition)
        {
            byte[] result = null;

            long bufSize = length * ReferenceData.WaveFormat.nBlockAlign;

            try
            {
                BeginInput();

                // ǂݍ݊JnʒuZo
                int start = Math.Min(
                    (int)(time * SampleRate),
                    _plugin.MediaFileInfo.InputAudioSampleCount - length);

                // f[^擾
                CacheManager.Manager.DemandTemporalMemory(bufSize);
                result = _plugin.ReadMediaFileAudio(start, length);
            }
            finally
            {
                EndInput();

                // 擾łȂȂ΃[߃f[^쐬
                if (result == null)
                {
                    CacheManager.Manager.DemandTemporalMemory(bufSize);
                    result = new byte[bufSize];
                    Array.Clear(result, 0, result.Length);
                }

                // o̓I[fBIQƃLbVɒǉ
                CacheManager.Manager.AddWeakCache(result, result.Length);
            }

            return result;
        }

        #endregion

        #region InputBase o

        public override string CorrespondingType
        {
            get { return "fBAt@C (*.*)|*.*"; }
        }

        public override string FileName
        {
            get { return (_plugin == null) ? "" : _plugin.MediaFilePath; }
        }

        public override void Initialize()
        {
            // gpȂ͂
        }

        public override bool Load(string file)
        {
            // gpȂ͂
            return false;
        }

        public override InputChild[] GetChildInput()
        {
            // gpȂ͂ꉞ
            return new InputChild[] { this };
        }

        public override object SaveInnerData()
        {
            return null;
        }

        public override void LoadInnerData(object data)
        {
            // Ȃ
        }

        #endregion

        #region PluginBase o

        public override string PluginName
        {
            get { return "AviUtl̓vOCp"; }
        }

        public override string Author
        {
            get { return "[`F"; }
        }

        public override string InfoLink
        {
            get { return "http://www.ruche-home.net/"; }
        }

        public override string Description
        {
            get
            {
                return "AviUtl̓̓vOC(*.aui)𗘗păfBAt@Cǂݍ݂܂B";
            }
        }

        #endregion

        #region IDisposable o

        public override void Dispose()
        {
            _plugin = null;
        }

        #endregion
    }
}
